from flask import Flask, render_template, request, redirect, flash, url_for
import mysql.connector
from mysql.connector import Error
import re

app = Flask(__name__)
app.secret_key = "your_secret_key"

# Database configuration
DB_CONFIG = {
    "host": "127.0.0.1",
    "user": "donald",
    "password": "donaldr12",
    "database": "iptv2"
}

def get_db_connection():
    return mysql.connector.connect(**DB_CONFIG)

def get_next_id():
    try:
        conn = get_db_connection()
        cursor = conn.cursor()
        cursor.execute("SELECT MAX(id) FROM Client")
        max_id = cursor.fetchone()[0]
        return max_id + 1 if max_id else 5201
    except Error as e:
        flash(f"Database error: {str(e)}", "danger")
        return None
    finally:
        if 'conn' in locals() and conn.is_connected():
            cursor.close()
            conn.close()

def validate_mac(mac):
    return re.match(r'^([0-9A-F]{2}:){5}[0-9A-F]{2}$', mac)

def add_client_to_livepackage(client_id):
    try:
        conn = mysql.connector.connect(**DB_CONFIG)
        cursor = conn.cursor()
        
        # 1. Get current clientIds from LivePackage (ID = 1)
        cursor.execute("SELECT clientIds FROM LivePackage WHERE id = 1")
        result = cursor.fetchone()
        
        if result:
            current_client_ids = result[0] or ""  # Handle NULL case
            client_ids_list = current_client_ids.split(',') if current_client_ids else []
            
            # 2. Add new client ID if not already present
            if str(client_id) not in client_ids_list:
                updated_client_ids = current_client_ids + ("," if current_client_ids else "") + str(client_id)
                
                # 3. Update LivePackage
                update_query = "UPDATE LivePackage SET clientIds = %s WHERE id = 1"
                cursor.execute(update_query, (updated_client_ids,))
                conn.commit()
                return True
        return False
    except Error as err:
        flash(f"❌ LivePackage update error: {err}", "danger")
        return False
    finally:
        if 'conn' in locals() and conn.is_connected():
            cursor.close()
            conn.close()

@app.route("/")
def index():
    search_query = request.args.get('search', '')
    try:
        conn = get_db_connection()
        cursor = conn.cursor(dictionary=True)
        
        if search_query:
            cursor.execute(
                "SELECT * FROM Client WHERE name LIKE %s OR ip LIKE %s OR mac LIKE %s ORDER BY id DESC",
                (f"%{search_query}%", f"%{search_query}%", f"%{search_query}%")
            )
        else:
            cursor.execute("SELECT * FROM Client ORDER BY id DESC")
            
        clients = cursor.fetchall()
        return render_template("index.html", clients=clients, search_query=search_query)
    except Error as e:
        flash(f"Database error: {str(e)}", "danger")
        return render_template("index.html", clients=[])
    finally:
        if 'conn' in locals() and conn.is_connected():
            cursor.close()
            conn.close()

@app.route("/add", methods=["POST"])
def add_client():
    name = request.form.get("name", "").strip().upper()
    ip = request.form.get("ip", "0.0.0.0").strip()
    mac = request.form.get("mac", "").strip().upper()
    
    if not name or not mac:
        flash("Name and MAC address are required", "danger")
        return redirect(url_for("index"))
    
    if not validate_mac(mac):
        flash("MAC address must be in XX:XX:XX:XX:XX:XX format", "danger")
        return redirect(url_for("index"))
    
    try:
        next_id = get_next_id()
        if not next_id:
            raise Error("Could not generate client ID")
            
        conn = get_db_connection()
        cursor = conn.cursor()
        cursor.execute(
            "INSERT INTO Client (id, name, ip, mac) VALUES (%s, %s, %s, %s)",
            (next_id, name, ip, mac)
        )
        conn.commit()
        
        # Add client to LivePackage
        if add_client_to_livepackage(next_id):
            flash(f"✅ Client added successfully to both Client and LivePackage", "success")
        else:
            flash(f"⚠️ Client added but failed to update LivePackage", "warning")
            
    except Error as e:
        flash(f"❌ Error adding client: {str(e)}", "danger")
    finally:
        if 'conn' in locals() and conn.is_connected():
            cursor.close()
            conn.close()
    
    return redirect(url_for("index"))

@app.route("/edit/<int:client_id>", methods=["GET", "POST"])
def edit_client(client_id):
    if request.method == "POST":
        name = request.form.get("name", "").strip().upper()
        mac = request.form.get("mac", "").strip().upper()
        
        if not validate_mac(mac):
            flash("❌ Invalid MAC address format", "danger")
            return redirect(url_for("edit_client", client_id=client_id))
        
        try:
            conn = get_db_connection()
            cursor = conn.cursor()
            cursor.execute(
                "UPDATE Client SET name = %s, mac = %s WHERE id = %s",
                (name, mac, client_id)
            )
            conn.commit()
            flash("✅ Client updated successfully", "success")
        except Error as e:
            flash(f"❌ Update failed: {str(e)}", "danger")
        finally:
            if 'conn' in locals() and conn.is_connected():
                cursor.close()
                conn.close()
        
        return redirect(url_for("index"))
    
    try:
        conn = get_db_connection()
        cursor = conn.cursor(dictionary=True)
        cursor.execute("SELECT * FROM Client WHERE id = %s", (client_id,))
        client = cursor.fetchone()
        
        if not client:
            flash("❌ Client not found", "danger")
            return redirect(url_for("index"))
            
        return render_template("edit.html", client=client)
    except Error as e:
        flash(f"❌ Database error: {str(e)}", "danger")
        return redirect(url_for("index"))
    finally:
        if 'conn' in locals() and conn.is_connected():
            cursor.close()
            conn.close()

@app.route("/delete/<int:client_id>")
def delete_client(client_id):
    try:
        conn = get_db_connection()
        cursor = conn.cursor()
        
        # Remove from LivePackage first
        cursor.execute("SELECT clientIds FROM LivePackage WHERE id = 1")
        result = cursor.fetchone()
        
        if result and result[0]:
            current_ids = result[0].split(',')
            updated_ids = ','.join([id for id in current_ids if id != str(client_id)])
            cursor.execute("UPDATE LivePackage SET clientIds = %s WHERE id = 1", (updated_ids,))
        
        # Then delete from Client
        cursor.execute("DELETE FROM Client WHERE id = %s", (client_id,))
        conn.commit()
        flash("✅ Client deleted successfully from both tables", "success")
    except Error as e:
        flash(f"❌ Delete failed: {str(e)}", "danger")
    finally:
        if 'conn' in locals() and conn.is_connected():
            cursor.close()
            conn.close()
    
    return redirect(url_for("index"))

if __name__ == "__main__":
    app.run(host="0.0.0.0", port=7700, debug=True)
